// Copyright © 2022 Nevis Security AG. All rights reserved.

import 'package:flutter/services.dart';
import 'package:nevis_mobile_authentication_sdk_platform_interface/nevis_mobile_authentication_sdk_platform_interface.dart';

/// iOS side implementation of [OperationsPlatformInterface].
class MethodChannelIosOperations extends OperationsPlatformInterface {
  final MethodChannel _channel;

  /// Default constructor for [MethodChannelIosOperations].
  MethodChannelIosOperations(this._channel);

  @override
  Future<void> outOfBandOperation({
    required String operationId,
    required String subOperationId,
    required OutOfBandPayload payload,
    required Function(OutOfBandPlatformOperationType) onOperationType,
    required Function(OutOfBandOperationError) onError,
  }) async {
    final operation = OutOfBandPlatformOperation(
      operationId: operationId,
      onOperationType: onOperationType,
      onErrorCallback: onError,
    );
    PlatformOperationCache.instance.createOperation(operation);
    Map<String, dynamic> message = OobOperationMessage(
      operationId: operationId,
      subOperationId: subOperationId,
      payload: payload,
    ).toJson();
    await _channel.invokeMethod(
      ChannelMethodNames.sdk.oobOperation,
      message,
    );
  }

  @override
  Future<void> outOfBandRegister({
    required String operationId,
    DeviceInformation? deviceInformation,
    bool? allowClass2AndroidSensors,
    bool? allowDevicePasscodeAsFallback,
    required AuthenticatorSelector authenticatorSelector,
    PinEnroller? pinEnroller,
    BiometricUserVerifier? biometricUserVerifier,
    FingerprintUserVerifier? fingerprintUserVerifier,
    required Function onSuccess,
    required Function(OperationError) onError,
  }) async {
    final registration = OutOfBandRegistrationPlatformOperation(
      operationId: operationId,
      authenticatorSelector: authenticatorSelector,
      pinEnroller: pinEnroller,
      biometricUserVerifier: biometricUserVerifier,
      fingerprintUserVerifier: fingerprintUserVerifier,
      onSuccessCallback: onSuccess,
      onErrorCallback: onError,
    );
    PlatformOperationCache.instance.createOperation(registration);
    Map<String, dynamic> message = OobRegistrationMessage(
      operationId: operationId,
      deviceInformation: deviceInformation,
      pinPolicy: pinEnroller?.pinPolicy,
      allowClass2AndroidSensors: allowClass2AndroidSensors,
      allowDevicePasscodeAsFallback: allowDevicePasscodeAsFallback,
    ).toJson();
    await _channel.invokeMethod(
      ChannelMethodNames.sdk.oobRegister,
      message,
    );
  }

  @override
  Future<void> outOfBandAuthenticate({
    required String operationId,
    AccountSelector? accountSelector,
    required AuthenticatorSelector authenticatorSelector,
    PinUserVerifier? pinUserVerifier,
    BiometricUserVerifier? biometricUserVerifier,
    FingerprintUserVerifier? fingerprintUserVerifier,
    required Function(AuthorizationProvider?) onSuccess,
    required Function(OperationError) onError,
  }) async {
    final authentication = OutOfBandAuthenticationPlatformOperation(
      operationId: operationId,
      accountSelector: accountSelector,
      authenticatorSelector: authenticatorSelector,
      pinUserVerifier: pinUserVerifier,
      biometricUserVerifier: biometricUserVerifier,
      fingerprintUserVerifier: fingerprintUserVerifier,
      onSuccessCallback: onSuccess,
      onErrorCallback: onError,
    );
    PlatformOperationCache.instance.createOperation(authentication);
    Map<String, dynamic> message = OobAuthenticationMessage(
      operationId: operationId,
    ).toJson();
    await _channel.invokeMethod(
      ChannelMethodNames.sdk.oobAuthenticate,
      message,
    );
  }

  @override
  Future<void> outOfBandPayloadDecode({
    required String operationId,
    String? json,
    String? base64UrlEncoded,
    required Function(OutOfBandPayload?) onSuccess,
    required Function(OutOfBandPayloadError) onError,
  }) async {
    final operation = OutOfBandPayloadDecodePlatformOperation(
      operationId: operationId,
      onSuccessCallback: onSuccess,
      onErrorCallback: onError,
    );
    PlatformOperationCache.instance.createOperation(operation);
    Map<String, dynamic> message = OobPayloadDecodeMessage(
      operationId: operationId,
      json: json,
      base64UrlEncoded: base64UrlEncoded,
    ).toJson();
    await _channel.invokeMethod(
      ChannelMethodNames.sdk.payloadDecode,
      message,
    );
  }

  @override
  Future<void> register({
    required String operationId,
    required String username,
    DeviceInformation? deviceInformation,
    AuthorizationProvider? authorizationProvider,
    bool? allowClass2AndroidSensors,
    bool? allowDevicePasscodeAsFallback,
    required AuthenticatorSelector authenticatorSelector,
    PinEnroller? pinEnroller,
    BiometricUserVerifier? biometricUserVerifier,
    FingerprintUserVerifier? fingerprintUserVerifier,
    required Function onSuccess,
    required Function(OperationError) onError,
  }) async {
    final operation = RegistrationPlatformOperation(
      operationId: operationId,
      authenticatorSelector: authenticatorSelector,
      pinEnroller: pinEnroller,
      biometricUserVerifier: biometricUserVerifier,
      fingerprintUserVerifier: fingerprintUserVerifier,
      onSuccessCallback: onSuccess,
      onErrorCallback: onError,
    );
    PlatformOperationCache.instance.createOperation(operation);
    Map<String, dynamic> message = RegistrationMessage(
      operationId: operationId,
      username: username,
      deviceInformation: deviceInformation,
      authorizationProvider: authorizationProvider,
      pinPolicy: pinEnroller?.pinPolicy,
      allowClass2AndroidSensors: allowClass2AndroidSensors,
      allowDevicePasscodeAsFallback: allowDevicePasscodeAsFallback,
    ).toJson();
    await _channel.invokeMethod(
      ChannelMethodNames.sdk.register,
      message,
    );
  }

  @override
  Future<void> authCloudApiRegister({
    required String operationId,
    String? enrollResponse,
    String? appLinkUri,
    DeviceInformation? deviceInformation,
    bool? allowClass2AndroidSensors,
    bool? allowDevicePasscodeAsFallback,
    required AuthenticatorSelector authenticatorSelector,
    PinEnroller? pinEnroller,
    BiometricUserVerifier? biometricUserVerifier,
    FingerprintUserVerifier? fingerprintUserVerifier,
    required Function onSuccess,
    required Function(AuthCloudApiError) onError,
  }) async {
    final operation = AuthCloudApiRegistrationPlatformOperation(
      operationId: operationId,
      authenticatorSelector: authenticatorSelector,
      pinEnroller: pinEnroller,
      biometricUserVerifier: biometricUserVerifier,
      fingerprintUserVerifier: fingerprintUserVerifier,
      onSuccessCallback: onSuccess,
      onErrorCallback: onError,
    );
    PlatformOperationCache.instance.createOperation(operation);
    Map<String, dynamic> message = AuthCloudApiRegistrationMessage(
      operationId: operationId,
      enrollResponse: enrollResponse,
      appLinkUri: appLinkUri,
      deviceInformation: deviceInformation,
      pinPolicy: pinEnroller?.pinPolicy,
      allowClass2AndroidSensors: allowClass2AndroidSensors,
      allowDevicePasscodeAsFallback: allowDevicePasscodeAsFallback,
    ).toJson();
    await _channel.invokeMethod(
      ChannelMethodNames.sdk.authCloudApiRegister,
      message,
    );
  }

  @override
  Future<void> authenticate({
    required String operationId,
    required String username,
    SessionProvider? sessionProvider,
    RetryPolicy? retryPolicy,
    required AuthenticatorSelector authenticatorSelector,
    PinUserVerifier? pinUserVerifier,
    BiometricUserVerifier? biometricUserVerifier,
    FingerprintUserVerifier? fingerprintUserVerifier,
    required Function(AuthorizationProvider?) onSuccess,
    required Function(OperationError) onError,
  }) async {
    final operation = AuthenticationPlatformOperation(
        operationId: operationId,
        authenticatorSelector: authenticatorSelector,
        biometricUserVerifier: biometricUserVerifier,
        fingerprintUserVerifier: fingerprintUserVerifier,
        pinUserVerifier: pinUserVerifier,
        onSuccessCallback: onSuccess,
        onErrorCallback: onError);
    PlatformOperationCache.instance.createOperation(operation);
    Map<String, dynamic> message = AuthenticationMessage(
      operationId: operationId,
      username: username,
      sessionProvider: sessionProvider,
      retryPolicy: retryPolicy,
    ).toJson();
    await _channel.invokeMethod(
      ChannelMethodNames.sdk.authenticate,
      message,
    );
  }

  @override
  Future<void> deregister({
    required String operationId,
    required String username,
    String? aaid,
    AuthorizationProvider? authorizationProvider,
    required Function onSuccess,
    required Function(OperationError) onError,
  }) async {
    final operation = DeregistrationPlatformOperation(
      operationId: operationId,
      onSuccessCallback: onSuccess,
      onErrorCallback: onError,
    );
    PlatformOperationCache.instance.createOperation(operation);
    Map<String, dynamic> message = DeregistrationMessage(
      operationId: operationId,
      username: username,
      aaid: aaid,
      authorizationProvider: authorizationProvider,
    ).toJson();
    await _channel.invokeMethod(
      ChannelMethodNames.sdk.deregister,
      message,
    );
  }

  @override
  Future<void> changePin({
    required String operationId,
    required String username,
    required PinChanger pinChanger,
    required Function onSuccess,
    required Function(PinChangeError) onError,
  }) async {
    final operation = ChangePinPlatformOperation(
      operationId: operationId,
      pinChanger: pinChanger,
      onSuccessCallback: onSuccess,
      onErrorCallback: onError,
    );
    PlatformOperationCache.instance.createOperation(operation);
    Map<String, dynamic> message = ChangePinMessage(
      operationId: operationId,
      username: username,
      pinPolicy: pinChanger.pinPolicy,
    ).toJson();
    await _channel.invokeMethod(
      ChannelMethodNames.sdk.changePin,
      message,
    );
  }

  @override
  Future<void> deviceInformationChange({
    required String operationId,
    String? name,
    String? fcmRegistrationToken,
    required bool disablePushNotifications,
    RetryPolicy? retryPolicy,
    required Function onSuccess,
    required Function(DeviceInformationChangeError) onError,
  }) async {
    final operation = DeviceInformationChangePlatformOperation(
      operationId: operationId,
      onSuccessCallback: onSuccess,
      onErrorCallback: onError,
    );
    PlatformOperationCache.instance.createOperation(operation);
    Map<String, dynamic> message = DeviceInformationChangeMessage(
      operationId: operationId,
      name: name,
      fcmRegistrationToken: fcmRegistrationToken,
      disablePushNotifications: disablePushNotifications,
      retryPolicy: retryPolicy,
    ).toJson();
    await _channel.invokeMethod(
      ChannelMethodNames.sdk.deviceInformationChange,
      message,
    );
  }
}
