// Copyright © 2022 Nevis Security AG. All rights reserved.

import 'package:flutter/services.dart';
import 'package:flutter_test/flutter_test.dart';
import 'package:mockito/mockito.dart';
import 'package:nevis_mobile_authentication_sdk_ios/nevis_mobile_authentication_sdk_ios.dart';
import 'package:nevis_mobile_authentication_sdk_platform_interface/nevis_mobile_authentication_sdk_platform_interface.dart';

import '../mocks/method_channel_test_shared_mocks.mocks.dart';
import '../test_data/change_pin_test_data.dart';
import '../test_data/common_test_data.dart';
import '../util/ios_test_init.dart';
import '../util/method_channel_native_call_mock.dart';

void main() {
  IosTestInit.initIosTestCase();

  group(NevisMobileAuthenticationSdkIos, () {
    const MethodChannel channel = MethodChannel(
      CommonTestData.methodChannelName,
    );

    group("Given changePin is called from the native side", () {
      const methodName = 'changePin';
      late MockMethodChannelMethodCallCache mockCache;

      group("When the incoming parameters are correct", () {
        setUp(() async {
          mockCache = MockMethodChannelMethodCallCache();
          NevisMobileAuthenticationSdkIos.mock(
              MethodChannelMethodCallHandler.mock(mockCache));
        });

        tearDown(() async {
          channel.setMockMethodCallHandler(null);
        });

        test("Then the call is propagated to the cached delegate", () async {
          final ChangePinPlatformOperation mockOperation =
              MockChangePinPlatformOperation();
          final MockPinChanger mockPinChanger = MockPinChanger();
          final PinChangeHandler mockPinChangeHandler = MockPinChangeHandler();
          final operationId = CommonTestData.operationId;
          when(mockCache.getOperation<ChangePinPlatformOperation>(operationId))
              .thenReturn(mockOperation);
          when(mockOperation.pinChanger).thenReturn(mockPinChanger);
          when(mockOperation.pinChangeHandler).thenReturn(mockPinChangeHandler);
          when(mockPinChanger.changePin(any, any)).thenReturn(null);

          await channel.invokeMockMethod(
            methodName,
            ChangePinInTestData.correctArguments,
          );

          verifyInOrder([
            mockCache.getOperation<ChangePinPlatformOperation>(operationId),
            mockOperation.pinChanger,
            mockPinChanger.changePin(any, any),
          ]);
        });
      });

      group("When the incoming parameter are not correct", () {
        setUp(() async {
          mockCache = MockMethodChannelMethodCallCache();
          NevisMobileAuthenticationSdkIos.mock(
              MethodChannelMethodCallHandler.mock(mockCache));
        });

        tearDown(() async {
          channel.setMockMethodCallHandler(null);
        });

        test("Then PlatformChannelException is thrown", () async {
          final ChangePinPlatformOperation mockOperation =
              MockChangePinPlatformOperation();
          final MockPinChanger mockPinChanger = MockPinChanger();
          final PinChangeHandler mockPinChangeHandler = MockPinChangeHandler();
          final operationId = CommonTestData.operationId;
          when(mockCache.getOperation<ChangePinPlatformOperation>(operationId))
              .thenReturn(mockOperation);
          when(mockOperation.pinChanger).thenReturn(mockPinChanger);
          when(mockOperation.pinChangeHandler).thenReturn(mockPinChangeHandler);
          when(mockPinChanger.changePin(any, any)).thenReturn(null);

          try {
            await channel.invokeMockMethod(
              methodName,
              ChangePinInTestData.wrongArguments,
            );
            fail('PlatformException expected');
          } on PlatformException catch (e) {
            expect(e.code, "error");
          } on Exception catch (_) {
            fail('PlatformException expected');
          }
          verifyNever(mockCache.getOperation(operationId));
        });
      });

      group("When the operationId is missing", () {
        setUp(() async {
          mockCache = MockMethodChannelMethodCallCache();
          NevisMobileAuthenticationSdkIos.mock(
              MethodChannelMethodCallHandler.mock(mockCache));
        });

        tearDown(() async {
          channel.setMockMethodCallHandler(null);
        });

        test("Then PlatformChannelException is thrown", () async {
          final operationId = CommonTestData.wrongId;
          when(mockCache.getOperation(operationId)).thenThrow(
              PlatformChannelException(
                  error: PlatformChannelError.noOperationFoundError(
                      operationId: operationId)));

          try {
            await channel.invokeMockMethod(
              methodName,
              ChangePinInTestData.wrongIdArguments,
            );
            fail('PlatformException expected');
          } on PlatformException catch (e) {
            expect(
              e.message,
              CommonTestData.platformChannelExceptionMatcher,
            );
          } on Exception catch (_) {
            fail('PlatformException expected');
          }
          verify(mockCache.getOperation(operationId));
        });
      });
    });
  });
}
