// Copyright © 2022 Nevis Security AG. All rights reserved.

import 'package:flutter/services.dart';
import 'package:flutter_test/flutter_test.dart';
import 'package:mockito/mockito.dart';
import 'package:nevis_mobile_authentication_sdk_ios/nevis_mobile_authentication_sdk_ios.dart';
import 'package:nevis_mobile_authentication_sdk_platform_interface/nevis_mobile_authentication_sdk_platform_interface.dart';

import '../mocks/method_channel_test_shared_mocks.mocks.dart';
import '../test_data/common_test_data.dart';
import '../test_data/enroll_pin_test_data.dart';
import '../util/ios_test_init.dart';
import '../util/method_channel_native_call_mock.dart';

void main() {
  IosTestInit.initIosTestCase();

  group(NevisMobileAuthenticationSdkIos, () {
    const MethodChannel channel = MethodChannel(
      CommonTestData.methodChannelName,
    );

    group("Given enrollPin is called from the native side", () {
      const methodName = 'enrollPin';
      late MockMethodChannelMethodCallCache mockCache;

      group("When the incoming parameters are correct", () {
        setUp(() async {
          mockCache = MockMethodChannelMethodCallCache();
          NevisMobileAuthenticationSdkIos.mock(
              MethodChannelMethodCallHandler.mock(mockCache));
        });

        tearDown(() async {
          channel.setMockMethodCallHandler(null);
        });

        test("Then the call is propagated to the cached delegate", () async {
          final MockOutOfBandRegistrationPlatformOperation mockOperation =
              MockOutOfBandRegistrationPlatformOperation();
          final updatedOperation = MockOutOfBandRegistrationPlatformOperation();
          final mockPinEnroller = MockPinEnroller();
          final mockPinEnrollmentHandler = MockPinEnrollmentHandler();
          final operationId = CommonTestData.operationId;
          when(mockCache
                  .getOperation<UserInteractionPlatformOperation>(operationId))
              .thenReturn(mockOperation);
          when(mockOperation.updateHandler(any)).thenReturn(updatedOperation);
          when(mockOperation.pinEnroller).thenReturn(mockPinEnroller);
          when(mockOperation.pinEnrollmentHandler)
              .thenReturn(mockPinEnrollmentHandler);
          when(mockPinEnroller.enrollPin(any, any)).thenReturn(null);

          await channel.invokeMockMethod(
            methodName,
            EnrollPinTestData.correctArguments,
          );

          verifyInOrder([
            mockCache
                .getOperation<UserInteractionPlatformOperation>(operationId),
            mockOperation.updateHandler(UserInteractionState.enroll),
            mockCache.updateOperation(
              operationId: operationId,
              operation: updatedOperation,
            ),
            mockPinEnroller.enrollPin(any, any),
          ]);
        });
      });

      group("When the incoming parameter are not correct", () {
        setUp(() async {
          mockCache = MockMethodChannelMethodCallCache();
          NevisMobileAuthenticationSdkIos.mock(
              MethodChannelMethodCallHandler.mock(mockCache));
        });

        tearDown(() async {
          channel.setMockMethodCallHandler(null);
        });

        test("Then PlatformChannelException is thrown", () async {
          final operationId = CommonTestData.operationId;
          final MockOutOfBandRegistrationPlatformOperation mockOperation =
              MockOutOfBandRegistrationPlatformOperation();
          final updatedOperation = MockOutOfBandRegistrationPlatformOperation();
          final mockPinEnroller = MockPinEnroller();
          final mockPinEnrollmentHandler = MockPinEnrollmentHandler();
          when(mockCache
                  .getOperation<UserInteractionPlatformOperation>(operationId))
              .thenReturn(mockOperation);
          when(mockOperation.updateHandler(any)).thenReturn(updatedOperation);
          when(mockOperation.pinEnroller).thenReturn(mockPinEnroller);
          when(mockOperation.pinEnrollmentHandler)
              .thenReturn(mockPinEnrollmentHandler);
          when(mockPinEnroller.enrollPin(any, any)).thenReturn(null);

          try {
            await channel.invokeMockMethod(
              methodName,
              EnrollPinTestData.wrongArguments,
            );
            fail('PlatformException expected');
          } on PlatformException catch (e) {
            expect(e.code, "error");
          } on Exception catch (_) {
            fail('PlatformException expected');
          }
          verifyNever(mockCache.getOperation(operationId));
        });
      });

      group("When the operationId is missing", () {
        setUp(() async {
          mockCache = MockMethodChannelMethodCallCache();
          NevisMobileAuthenticationSdkIos.mock(
              MethodChannelMethodCallHandler.mock(mockCache));
        });

        tearDown(() async {
          channel.setMockMethodCallHandler(null);
        });

        test("Then PlatformChannelException is thrown", () async {
          final operationId = CommonTestData.wrongId;
          when(mockCache.getOperation(operationId)).thenThrow(
              PlatformChannelException(
                  error: PlatformChannelError.noOperationFoundError(
                      operationId: operationId)));

          try {
            await channel.invokeMockMethod(
              methodName,
              EnrollPinTestData.wrongIdArguments,
            );
            fail('PlatformException expected');
          } on PlatformException catch (e) {
            expect(
              e.message,
              CommonTestData.platformChannelExceptionMatcher,
            );
          } on Exception catch (_) {
            fail('PlatformException expected');
          }
          verify(mockCache.getOperation(operationId));
        });
      });
    });
  });
}
