// Copyright © 2022 Nevis Security AG. All rights reserved.

import 'package:flutter/services.dart';
import 'package:flutter_test/flutter_test.dart';
import 'package:mockito/mockito.dart';
import 'package:nevis_mobile_authentication_sdk_ios/nevis_mobile_authentication_sdk_ios.dart';
import 'package:nevis_mobile_authentication_sdk_platform_interface/nevis_mobile_authentication_sdk_platform_interface.dart';

import '../mocks/method_channel_test_shared_mocks.mocks.dart';
import '../test_data/common_test_data.dart';
import '../test_data/verify_user_test_data.dart';
import '../util/ios_test_init.dart';
import '../util/method_channel_native_call_mock.dart';

void main() {
  IosTestInit.initIosTestCase();

  group(NevisMobileAuthenticationSdkIos, () {
    const MethodChannel channel = MethodChannel(
      CommonTestData.methodChannelName,
    );

    group("Given verifyUser is called from the native side", () {
      const methodName = 'verifyUser';
      late MockMethodChannelMethodCallCache mockCache;
      group("When the incoming parameters are correct and for pin verification",
          () {
        setUp(() async {
          mockCache = MockMethodChannelMethodCallCache();
          NevisMobileAuthenticationSdkIos.mock(
              MethodChannelMethodCallHandler.mock(mockCache));
        });

        tearDown(() async {
          channel.setMockMethodCallHandler(null);
        });

        test("Then the call is propagated to the cached delegate", () async {
          final MockAuthenticationPlatformOperation mockOperation =
              MockAuthenticationPlatformOperation();
          final operationId = CommonTestData.operationId;
          when(mockCache.getOperation(operationId)).thenReturn(mockOperation);
          when(mockCache.updateOperation(
            operationId: operationId,
            operation: anyNamed("operation"),
          )).thenReturn(null);
          when(mockCache.removeOperation(operationId)).thenReturn(null);
          when(mockOperation.copyWith(
            userVerificationHandler: anyNamed("userVerificationHandler"),
            authenticatorSelectionHandler:
                anyNamed("authenticatorSelectionHandler"),
          )).thenReturn(mockOperation);

          await channel.invokeMockMethod(
            methodName,
            VerifyUserTestData.verifyPinCorrectArguments,
          );

          verifyInOrder([
            mockCache.getOperation(operationId),
            mockCache.updateOperation(
              operationId: operationId,
              operation: anyNamed("operation"),
            ),
            mockOperation.verifyUser(
              context: anyNamed("context"),
              handler: anyNamed("handler"),
              aaid: anyNamed("aaid"),
            ),
          ]);
        });
      });

      group(
          "When the incoming parameters are not correct and for pin verification",
          () {
        setUp(() async {
          mockCache = MockMethodChannelMethodCallCache();
          NevisMobileAuthenticationSdkIos.mock(
              MethodChannelMethodCallHandler.mock(mockCache));
        });

        tearDown(() async {
          channel.setMockMethodCallHandler(null);
        });

        test("Then PlatformChannelException is thrown", () async {
          final AuthenticationPlatformOperation mockOperation =
              MockAuthenticationPlatformOperation();
          final operationId = CommonTestData.operationId;
          when(mockCache.getOperation(operationId)).thenReturn(mockOperation);
          when(mockCache.updateOperation(
            operationId: operationId,
            operation: anyNamed("operation"),
          )).thenReturn(null);
          when(mockCache.removeOperation(operationId)).thenReturn(null);
          when(mockOperation.copyWith(
            userVerificationHandler: anyNamed("userVerificationHandler"),
            authenticatorSelectionHandler:
                anyNamed("authenticatorSelectionHandler"),
          )).thenReturn(mockOperation);

          try {
            await channel.invokeMockMethod(
              methodName,
              VerifyUserTestData.verifyPinWrongArguments,
            );
            fail('PlatformException expected');
          } on PlatformException catch (e) {
            expect(e.code, "error");
          } on Exception catch (_) {
            fail('PlatformException expected');
          }
          verifyNever(mockCache.getOperation(operationId));
        });
      });

      group(
          "When the incoming parameters are correct and for fingerprint verification",
          () {
        setUp(() async {
          mockCache = MockMethodChannelMethodCallCache();
          NevisMobileAuthenticationSdkIos.mock(
              MethodChannelMethodCallHandler.mock(mockCache));
        });

        tearDown(() async {
          channel.setMockMethodCallHandler(null);
        });

        test("Then the call is propagated to the cached delegate", () async {
          final MockAuthenticationPlatformOperation mockOperation =
              MockAuthenticationPlatformOperation();
          final operationId = CommonTestData.operationId;
          when(mockCache.getOperation(operationId)).thenReturn(mockOperation);
          when(mockCache.updateOperation(
            operationId: operationId,
            operation: anyNamed("operation"),
          )).thenReturn(null);
          when(mockCache.removeOperation(operationId)).thenReturn(null);
          when(mockOperation.copyWith(
            userVerificationHandler: anyNamed("userVerificationHandler"),
            authenticatorSelectionHandler:
                anyNamed("authenticatorSelectionHandler"),
          )).thenReturn(mockOperation);

          await channel.invokeMockMethod(
            methodName,
            VerifyUserTestData.verifyFingerprintCorrectArguments,
          );

          verifyInOrder([
            mockCache.getOperation(operationId),
            mockCache.updateOperation(
              operationId: operationId,
              operation: anyNamed("operation"),
            ),
            mockOperation.verifyUser(
              context: anyNamed("context"),
              handler: anyNamed("handler"),
              aaid: anyNamed("aaid"),
            ),
          ]);
        });
      });

      group(
          "When the incoming parameters are not correct and for fingerprint verification",
          () {
        setUp(() async {
          mockCache = MockMethodChannelMethodCallCache();
          NevisMobileAuthenticationSdkIos.mock(
              MethodChannelMethodCallHandler.mock(mockCache));
        });

        tearDown(() async {
          channel.setMockMethodCallHandler(null);
        });

        test("Then PlatformChannelException is thrown", () async {
          final AuthenticationPlatformOperation mockOperation =
              MockAuthenticationPlatformOperation();
          final operationId = CommonTestData.operationId;
          when(mockCache.getOperation(operationId)).thenReturn(mockOperation);
          when(mockCache.updateOperation(
            operationId: operationId,
            operation: anyNamed("operation"),
          )).thenReturn(null);
          when(mockCache.removeOperation(operationId)).thenReturn(null);
          when(mockOperation.copyWith(
            userVerificationHandler: anyNamed("userVerificationHandler"),
            authenticatorSelectionHandler:
                anyNamed("authenticatorSelectionHandler"),
          )).thenReturn(mockOperation);

          try {
            await channel.invokeMockMethod(
              methodName,
              VerifyUserTestData.verifyFingerprintWrongArguments,
            );
            fail('PlatformException expected');
          } on PlatformException catch (e) {
            expect(e.code, "error");
          } on Exception catch (_) {
            fail('PlatformException expected');
          }
          verifyNever(mockCache.getOperation(operationId));
        });
      });

      group(
          "When the incoming parameters are correct and for biometric verification",
          () {
        setUp(() async {
          mockCache = MockMethodChannelMethodCallCache();
          NevisMobileAuthenticationSdkIos.mock(
              MethodChannelMethodCallHandler.mock(mockCache));
        });

        tearDown(() async {
          channel.setMockMethodCallHandler(null);
        });

        test("Then the call is propagated to the cached delegate", () async {
          final MockAuthenticationPlatformOperation mockOperation =
              MockAuthenticationPlatformOperation();
          final operationId = CommonTestData.operationId;
          when(mockCache.getOperation(operationId)).thenReturn(mockOperation);
          when(mockCache.updateOperation(
            operationId: operationId,
            operation: anyNamed("operation"),
          )).thenReturn(null);
          when(mockCache.removeOperation(operationId)).thenReturn(null);
          when(mockOperation.copyWith(
            userVerificationHandler: anyNamed("userVerificationHandler"),
            authenticatorSelectionHandler:
                anyNamed("authenticatorSelectionHandler"),
          )).thenReturn(mockOperation);

          await channel.invokeMockMethod(
            methodName,
            VerifyUserTestData.verifyBiometricCorrectArguments,
          );

          verifyInOrder([
            mockCache.getOperation(operationId),
            mockCache.updateOperation(
              operationId: operationId,
              operation: anyNamed("operation"),
            ),
            mockOperation.verifyUser(
              context: anyNamed("context"),
              handler: anyNamed("handler"),
              aaid: anyNamed("aaid"),
            ),
          ]);
        });
      });

      group(
          "When the incoming parameters are not correct and for biometric verification",
          () {
        setUp(() async {
          mockCache = MockMethodChannelMethodCallCache();
          NevisMobileAuthenticationSdkIos.mock(
              MethodChannelMethodCallHandler.mock(mockCache));
        });

        tearDown(() async {
          channel.setMockMethodCallHandler(null);
        });

        test("Then PlatformChannelException is thrown", () async {
          final AuthenticationPlatformOperation mockOperation =
              MockAuthenticationPlatformOperation();
          final operationId = CommonTestData.operationId;
          when(mockCache.getOperation(operationId)).thenReturn(mockOperation);
          when(mockCache.updateOperation(
            operationId: operationId,
            operation: anyNamed("operation"),
          )).thenReturn(null);
          when(mockCache.removeOperation(operationId)).thenReturn(null);
          when(mockOperation.copyWith(
            userVerificationHandler: anyNamed("userVerificationHandler"),
            authenticatorSelectionHandler:
                anyNamed("authenticatorSelectionHandler"),
          )).thenReturn(mockOperation);

          try {
            await channel.invokeMockMethod(
              methodName,
              VerifyUserTestData.verifyBiometricWrongArguments,
            );
            fail('PlatformException expected');
          } on PlatformException catch (e) {
            expect(e.code, "error");
          } on Exception catch (_) {
            fail('PlatformException expected');
          }
          verifyNever(mockCache.getOperation(operationId));
        });
      });

      group("When the operationId is missing", () {
        setUp(() async {
          mockCache = MockMethodChannelMethodCallCache();
          NevisMobileAuthenticationSdkIos.mock(
              MethodChannelMethodCallHandler.mock(mockCache));
        });

        tearDown(() async {
          channel.setMockMethodCallHandler(null);
        });

        test("Then PlatformChannelException is thrown", () async {
          final operationId = CommonTestData.wrongId;
          final UserInteractionPlatformOperation mockOperation =
              MockUserInteractionPlatformOperation();

          when(mockCache
                  .getOperation<UserInteractionPlatformOperation>(operationId))
              .thenReturn(mockOperation);
          when(mockOperation.copyWith(
            userVerificationHandler: anyNamed("userVerificationHandler"),
            authenticatorSelectionHandler:
                anyNamed("authenticatorSelectionHandler"),
          )).thenReturn(mockOperation);
          when(mockCache.updateOperation(
            operationId: operationId,
            operation: anyNamed("operation"),
          )).thenThrow(PlatformChannelException(
              error: PlatformChannelError.noOperationFoundError(
                  operationId: operationId)));

          try {
            await channel.invokeMockMethod(
              methodName,
              VerifyUserTestData.wrongIdArguments,
            );
            fail('PlatformException expected');
          } on PlatformException catch (e) {
            expect(
              e.message,
              CommonTestData.platformChannelExceptionMatcher,
            );
          } on Exception {
            fail('PlatformException expected');
          }
          verify(
            mockCache.updateOperation(
              operationId: operationId,
              operation: anyNamed("operation"),
            ),
          );
        });
      });
    });
  });
}
